import * as _codemirror_autocomplete from '@codemirror/autocomplete';
import { LanguageSupport } from '@codemirror/language';
import { Diagnostic } from '@codemirror/lint';
import { Extension, Transaction } from '@codemirror/state';
import { EditorView } from '@codemirror/view';
import { Properties } from 'csstype';
import { MapStore } from 'nanostores';

/**
 * Actions can return an object containing the two properties defined in this interface. Both are optional.
 * - update: An action can have a parameter. This method will be called whenever that parameter changes,
 *   immediately after Svelte has applied updates to the markup.
 * - destroy: Method that is called after the element is unmounted
 *
 * Additionally, you can specify which additional attributes and events the action enables on the applied element.
 * This applies to TypeScript typings only and has no effect at runtime.
 *
 * Example usage:
 * ```ts
 * interface Attributes {
 * 	newprop?: string;
 *	'on:event': (e: CustomEvent<boolean>) => void;
 * }
 *
 * export function myAction(node: HTMLElement, parameter: Parameter): ActionReturn<Parameter, Attributes> {
 *   // ...
 *   return {
 *     update: (updatedParameter) => {...},
 *     destroy: () => {...}
 *   };
 * }
 * ```
 *
 * Docs: https://svelte.dev/docs#template-syntax-element-directives-use-action
 */
interface ActionReturn<Parameter = any, Attributes extends Record<string, any> = Record<never, any>> {
    update?: (parameter: Parameter) => void;
    destroy?: () => void;
    /**
     * ### DO NOT USE THIS
     * This exists solely for type-checking and has no effect at runtime.
     * Set this through the `Attributes` generic instead.
     */
    $$_attributes?: Attributes;
}

type MaybePromise<T> = T | Promise<T>;
type Styles = {
    [val: string]: Properties | Styles;
};
type NeoCodemirrorOptions = {
    /**
     * Value of the editor. Required
     *
     * @example
     * ```svelte
     * <div use:codemirror={{ value: `let code = 'wow'` }} />
     * ```
     */
    value: string;
    /**
     * The editor setup to apply. Can be either `basic` or `minimal`.
     * Defaults to no setup
     *
     * @default undefined
     *
     * @example
     * ```svelte
     * <div use:codemirror={{ setup: 'minimal' }} />
     * ```
     *
     * @see https://codemirror.net/docs/ref/#codemirror.basicSetup
     * @see https://codemirror.net/docs/ref/#codemirror.minimalSetup
     */
    setup?: 'basic' | 'minimal';
    /**
     * The language to use. Can be either a `LanguageSupport` or a string.
     * Defaults to none.
     *
     * When it is a `LanguageSupport`, it will be passed directly to codemirror. However, when it is a string, it will be used to get the language support from the `langMap` option.
     *
     * @see langMap option
     *
     * @default undefined
     *
     * @example
     * ```svelte
     * <script>
     * 	import { javascript } from '@codemirror/lang-javascript';
     * </script>
     *
     * <div use:codemirror={{ lang: javascript() }} />
     * ```
     *
     * @example
     * ```svelte
     * <script>
     * 	import { javascript } from '@codemirror/lang-javascript';
     * 	import { html } from '@codemirror/lang-html';
     * </script>
     *
     * <div
     *   use:codemirror={{
     *     lang: 'html',
     *     langMap: {
     *       html: () => html(),
     *       js: () => javascript({ typescript: true })
     *     }
     *   }}
     * />
     * ```
     */
    lang?: LanguageSupport | string;
    /**
     * A map of language names to functions that return a `LanguageSupport`. Can be promises too.
     *
     * A use case would be having an instance of codemirror
     * that switches its language based on file type chosen. In that case, combining with dynamic imports can be a great performance boost.
     *
     * @default undefined
     *
     * @example
     * ```svelte
     * <div
     *   use:codemirror={{
     *     lang: 'html',
     *     langMap: {
     *       html: () => import('@codemirror/lang-html').then((m) => m html()),
     *       js: () => import('@codemirror/lang-javascript').then((m) => m javascript({ typescript: true }))
     *     }
     *   }}
     * />
     * ```
     */
    langMap?: Record<string, () => MaybePromise<LanguageSupport>>;
    /**
     * Whether to use tabs or spaces. Defaults to spaces.
     *
     * @default false
     *
     * @example
     * ```svelte
     * <div use:codemirror={{ useTabs: true }} />
     * ```
     *
     * @see https://codemirror.net/docs/ref/#commands.indentWithTab
     */
    useTabs?: boolean;
    /**
     * The size of a tab in spaces. Defaults to 2.
     *
     * @default 2
     *
     * @see https://codemirror.net/docs/ref/#state.EditorState^tabSize
     *
     * @example
     * ```svelte
     * <div use:codemirror={{ tabSize: 4 }} />
     * ```
     */
    tabSize?: number;
    /**
     * Whether to open the editor in readonly mode. Note its different from `editable`, which allows you to focus cursor in editor, but not make any changes.
     * Defaults to false.
     *
     * @default false
     *
     * @example
     * ```svelte
     * <div use:codemirror={{ readonly: true }} />
     * ```
     *
     * @see https://codemirror.net/docs/ref/#state.EditorState^readOnly
     */
    readonly?: boolean;
    /**
     * Cursor Position. If not specified, defaults to the start of the document.
     *
     * @default undefined
     *
     * @example
     * ```svelte
     * <div use:codemirror={{ cursorPos: 10 }} />
     * ```
     */
    cursorPos?: number;
    /**
     * Whether to autocomplete the language's basics
     *
     * @default true
     *
     * @example
     * ```svelte
     * <div use:codemirror={{ autocomplete: false }} />
     * ```
     */
    autocomplete?: boolean | Parameters<typeof _codemirror_autocomplete.autocompletion>[0];
    /**
     * Styles to pass to EditorView.theme. Defaults to none.
     *
     * @default undefined
     *
     * @example
     * ```svelte
     * <div use:codemirror={{ styles: { '.cm-scroller': { overflow: 'hidden' } } }} />
     * ```
     *
     * @see https://codemirror.net/6/docs/ref/#view.EditorView^theme
     */
    styles?: Styles;
    /**
     * The theme to use. Of type `Extension`. Defaults to none.
     *
     * @default undefined
     *
     * @example
     * ```svelte
     * <script>
     * 	import { oneDark } from '@codemirror/theme-one-dark';
     * </script>
     *
     * <div use:codemirror={{ theme: oneDark }} />
     * ```
     */
    theme?: Extension;
    /**
     * Diagnostics data to pass to the editor. Defaults to none.
     *
     * @default undefined
     *
     * @example
     * ```svelte
     * <script>
     * 	import { javascript } from '@codemirror/lang-javascript';
     *
     * 	const diagnostics = [
     * 		{
     * 			from: 0,
     * 			to: 10,
     * 			message: 'This is a diagnostic message',
     * 			severity: 'error'
     * 		}
     * 	];
     *
     * 	const lang = javascript({ typescript: true });
     * </script>
     *
     * <div use:codemirror={{ lang, diagnostics }} />
     * ```
     *
     * @see https://codemirror.net/docs/ref/#lint
     */
    diagnostics?: Diagnostic[];
    /**
     * The extensions to use. Defaults to empty array.
     *
     * @default []
     *
     * @example
     * ```svelte
     * <script>
     * 	import { closeBrackets } from '@codemirror/autocomplete';
     * 	import { bracketMatching, codeFolding } from '@codemirror/language';
     * </script>
     *
     * <div use:codemirror={{ extensions: [closeBrackets(), bracketMatching(), codeFolding()] }} />
     * ```
     */
    extensions?: Extension[];
    /**
     * Instance store passed to the editor. This is created with `withCodemirrorInstance` function. It lets you track any and all state changes.
     *
     * @default undefined
     *
     * @example
     * ```svelte
     * <script>
     * 	import { withCodemirrorInstance } from '@neocodemirror/svelte';
     *
     * 	const instanceStore = withCodemirrorInstance();
     *
     * $: console.log($instanceStore);
     * </script>
     *
     * <div use:codemirror={{ instanceStore }} />
     * ```
     */
    instanceStore?: MapStore<CodemirrorInstance>;
    /**
     * Triggers every time value of code editor is changed
     *
     * @default undefined
     *
     * @example
     * ```svelte
     * <div use:codemirror={{ onValueChange: (value) => console.log(value) }} />
     * ```
     */
    onTextChange?: (value: string) => void;
    /**
     * Triggers on any change in editor state. This includes changes in value, cursor position, diagnostics, etc.
     * The transaction object is exposed to the callback function.
     *
     * Note: If you are new to codemirror 6, this will trigger more than you probably think it will.
     *
     * @default undefined
     *
     * @example
     * ```svelte
     * <div use:codemirror={{ onChange: (transaction) => console.log(transaction.selection) }} />
     * ```
     */
    onChange?: (e: Transaction) => void;
};
type CodemirrorInstance = {
    view: EditorView | null;
    extensions: Extension | null;
    value: string | null;
};
declare const withCodemirrorInstance: () => MapStore<CodemirrorInstance>;
declare const codemirror: (node: HTMLElement, options: NeoCodemirrorOptions) => ActionReturn<NeoCodemirrorOptions, {
    'on:cmTextChange'?: ((e: CustomEvent<string>) => void) | undefined;
    'on:cmChange'?: ((e: CustomEvent<Transaction>) => void) | undefined;
}>;

export { NeoCodemirrorOptions, codemirror, withCodemirrorInstance };
